import itertools
import random

RNG = random.SystemRandom()


def sample_with_replacement(population, k):
    """"
    Return a random sample of size k from the population.

    Written a little unintuitively for speed purposes.
    """
    n = len(population)
    _random, _int = RNG.random, int
    return [population[_int(_random() * n)] for _ in itertools.repeat(None, k)]



import argparse

from pwdgen import password, ascii, alphanumeric, passphrase, numeric

generators = {
    'password': password,
    'ascii': ascii,
    'alphanumeric': alphanumeric,
    'passphrase': passphrase,
    'numeric': numeric
}


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("length", nargs='?', default=None)
    parser.add_argument("type", nargs='?', default='ascii')
    args = parser.parse_args()
    if args.length:
        print(generators[args.type](int(args.length)))
    else:
        for gen in [ascii, alphanumeric, passphrase, numeric]:
            print()
            print(gen())
        print()


if __name__ == '__main__':
    main()



import hashlib
import os
import tempfile
from pathlib import Path

import requests

WORDLISTS_PATH = Path(tempfile.gettempdir()) / Path('py-pwgen') / Path('wordlists')
DEFAULT_WORDLIST = 'https://github.com/brycedrennan/wordlists/raw/master/wordlists/archive-org-till-1830/wordlist_match1.txt'


def get_wordlist_path(url=DEFAULT_WORDLIST):
    print('ran')
    url_hash = hashlib.md5(url.encode('utf8')).hexdigest()
    wordlist_downloaded_path = WORDLISTS_PATH / Path(url_hash + '.txt')
    if not wordlist_downloaded_path.exists():
        print("\nDownloading wordlist from %s\n" % url)
        download_file(url, wordlist_downloaded_path)
    return wordlist_downloaded_path


def download_file(url, dest_path):
    os.makedirs(str(dest_path.parent), exist_ok=True)
    with dest_path.open('wb') as f:
        response = requests.get(url, stream=True)
        if not response.ok:
            raise Exception('Download Failed')

        for block in response.iter_content(1024):
            f.write(block)


class LazyString(object):
    def __init__(self, string_generator):
        self.string_generator = string_generator

    def __str__(self):
        return self.string_generator()



import string
from functools import partial

from randomlineaccess import IndexedOpen

from pwdgen.wordlist import LazyString
from .rand import sample_with_replacement
from .wordlist import get_wordlist_path

ASCII_CHARACTERS = string.ascii_letters + string.digits + string.punctuation
ALPHANUMERIC_CHARACTERS = string.ascii_letters + string.digits
UNICODE_SYMBOLS = u"☺★✰✓ÆÖÛ÷ɣɥΘЖ∭☀☂☃☎☻♫✈❤"
ALL_CHARACTERS = ASCII_CHARACTERS + UNICODE_SYMBOLS
_PASSPHRASE_CHARACTERS_TO_REMOVE = {ord(c): None for c in u"'\n\r"}


def password(length=100, characters=ALL_CHARACTERS, join_char=""):
    return join_char.join(sample_with_replacement(characters, length))


def passphrase(length=8, wordlist=LazyString(get_wordlist_path())):
    with IndexedOpen(str(wordlist)) as wordlist_file:
        return password(length, wordlist_file, join_char='-').translate(_PASSPHRASE_CHARACTERS_TO_REMOVE)


ascii = partial(password, characters=ASCII_CHARACTERS)
alphanumeric = partial(password, characters=ALPHANUMERIC_CHARACTERS)
numeric = partial(password, characters=string.digits)
symbols = partial(password, characters=UNICODE_SYMBOLS)



from setuptools import setup

__version__ = '1.62'

long_description = """
Quickly generate secure passwords and passphrases.
"""

setup(
    name="pwdgen",
    packages=["pwdgen"],
    version=__version__,
    description="Quickly generate secure passwords and passphrases.",
    author="Bryce Drennan",
    author_email="pwdgen@brycedrennan.org",
    url="https://github.com/brycedrennan/pwdgen",
    download_url='https://github.com/brycedrennan/pwdgen/tarball/' + __version__,
    keywords=["password", "passphrase"],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Security",
        "Topic :: Utilities",
    ],
    license='MIT',
    long_description=long_description,
    install_requires=['randomlineaccess==1.04', 'requests>=2.10.0'],
    entry_points={
        'console_scripts': ['pwdgen=pwdgen.command_line:main'],
    }
)
